@@ -43,6 +43,7 @@
 #include "wocky-signals-marshal.h"
 #include "wocky-utils.h"
 #include "wocky-c2s-porter.h"
+#include "wocky-meta-porter.h"
 
 G_DEFINE_TYPE (WockySession, wocky_session, G_TYPE_OBJECT)
 
@@ -149,9 +150,10 @@ wocky_session_constructed (GObject *object)
   WockySession *self = WOCKY_SESSION (object);
   WockySessionPrivate *priv = self->priv;
 
-  g_assert (priv->connection != NULL);
-
-  priv->porter = wocky_c2s_porter_new (priv->connection, priv->full_jid);
+  if (priv->connection != NULL)
+    priv->porter = wocky_c2s_porter_new (priv->connection, priv->full_jid);
+  else
+    priv->porter = wocky_meta_porter_new (priv->full_jid, priv->contact_factory);
 }
 
 static void
@@ -165,7 +167,12 @@ wocky_session_dispose (GObject *object)
 
   priv->dispose_has_run = TRUE;
 
-  g_object_unref (priv->connection);
+  if (priv->connection != NULL)
+    {
+      g_object_unref (priv->connection);
+      priv->connection = NULL;
+    }
+
   g_object_unref (priv->porter);
   g_object_unref (priv->contact_factory);
 
@@ -226,15 +233,26 @@ wocky_session_class_init (WockySessionClass *wocky_session_class)
 }
 
 WockySession *
-wocky_session_new (WockyXmppConnection *conn,
+wocky_session_new_with_connection (WockyXmppConnection *conn,
     const gchar *full_jid)
 {
+  g_return_val_if_fail (WOCKY_IS_XMPP_CONNECTION (conn), NULL);
+  g_return_val_if_fail (full_jid != NULL, NULL);
+
   return g_object_new (WOCKY_TYPE_SESSION,
       "connection", conn,
       "full-jid", full_jid,
       NULL);
 }
 
+WockySession *
+wocky_session_new_ll (const gchar *full_jid)
+{
+  return g_object_new (WOCKY_TYPE_SESSION,
+      "full-jid", full_jid,
+      NULL);
+}
+
 void wocky_session_start (WockySession *self)
 {
   WockySessionPrivate *priv = self->priv;
@@ -257,3 +275,19 @@ wocky_session_get_contact_factory (WockySession *self)
 
   return priv->contact_factory;
 }
+
+void
+wocky_session_set_jid (WockySession *self,
+    const gchar *jid)
+{
+  WockySessionPrivate *priv = self->priv;
+
+  g_free (priv->full_jid);
+  priv->full_jid = g_strdup (jid);
+
+  if (WOCKY_IS_META_PORTER (priv->porter))
+    {
+      wocky_meta_porter_set_jid (WOCKY_META_PORTER (priv->porter),
+          priv->full_jid);
+    }
+}
