@@ -69,7 +69,9 @@ GType wocky_session_get_type (void);
   (G_TYPE_INSTANCE_GET_CLASS ((obj), WOCKY_TYPE_SESSION, \
    WockySessionClass))
 
-WockySession * wocky_session_new (WockyXmppConnection *conn,
+WockySession * wocky_session_new_ll (const gchar *full_jid);
+
+WockySession * wocky_session_new_with_connection (WockyXmppConnection *conn,
     const gchar *full_jid);
 
 void wocky_session_start (WockySession *session);
@@ -78,6 +80,8 @@ WockyPorter * wocky_session_get_porter (WockySession *session);
 
 WockyContactFactory * wocky_session_get_contact_factory (WockySession *session);
 
+void wocky_session_set_jid (WockySession *session, const gchar *jid);
+
 G_END_DECLS
 
 #endif /* #ifndef __WOCKY_SESSION_H__*/
