@@ -228,6 +228,30 @@ wocky_xmpp_connection_dispose (GObject *object)
       priv->writer = NULL;
     }
 
+  if (priv->output_result != NULL)
+  {
+    g_object_unref (priv->output_result);
+    priv->output_result = NULL;
+  }
+
+  if (priv->output_cancellable != NULL)
+  {
+    g_object_unref (priv->output_cancellable);
+    priv->output_cancellable = NULL;
+  }
+
+  if (priv->input_result != NULL)
+  {
+    g_object_unref (priv->input_result);
+    priv->input_result = NULL;
+  }
+
+  if (priv->input_cancellable != NULL)
+  {
+    g_object_unref (priv->input_cancellable);
+    priv->input_cancellable = NULL;
+  }
+
   /* release any references held by the object here */
 
   if (G_OBJECT_CLASS (wocky_xmpp_connection_parent_class)->dispose)
@@ -304,6 +328,9 @@ finished:
   {
     GSimpleAsyncResult *r = priv->output_result;
 
+    if (priv->output_cancellable != NULL)
+      g_object_unref (priv->output_cancellable);
+
     priv->output_cancellable = NULL;
     priv->output_result = NULL;
 
@@ -374,9 +401,11 @@ wocky_xmpp_connection_send_open_async (WockyXmppConnection *connection,
   g_assert (priv->output_cancellable == NULL);
 
   priv->output_result = g_simple_async_result_new (G_OBJECT (connection),
-    callback, user_data, wocky_xmpp_connection_send_open_finish);
+    callback, user_data, wocky_xmpp_connection_send_open_async);
+
+  if (cancellable != NULL)
+    priv->output_cancellable = g_object_ref (cancellable);
 
-  priv->output_cancellable = cancellable;
   priv->offset = 0;
   priv->length = 0;
 
@@ -431,7 +460,7 @@ wocky_xmpp_connection_send_open_finish (WockyXmppConnection *connection,
     return FALSE;
 
   g_return_val_if_fail (g_simple_async_result_is_valid (result,
-    G_OBJECT (connection), wocky_xmpp_connection_send_open_finish), FALSE);
+    G_OBJECT (connection), wocky_xmpp_connection_send_open_async), FALSE);
 
   priv->output_open = TRUE;
 
@@ -523,6 +552,9 @@ finished:
   {
     GSimpleAsyncResult *r = priv->input_result;
 
+    if (priv->input_cancellable != NULL)
+      g_object_unref (priv->input_cancellable);
+
     priv->input_cancellable = NULL;
     priv->input_result = NULL;
 
@@ -565,9 +597,10 @@ wocky_xmpp_connection_recv_open_async (WockyXmppConnection *connection,
   g_assert (priv->input_cancellable == NULL);
 
   priv->input_result = g_simple_async_result_new (G_OBJECT (connection),
-    callback, user_data, wocky_xmpp_connection_recv_open_finish);
+    callback, user_data, wocky_xmpp_connection_recv_open_async);
 
-  priv->input_cancellable = cancellable;
+  if (cancellable != NULL)
+    priv->input_cancellable = g_object_ref (cancellable);
 
   wocky_xmpp_connection_do_read (connection);
 
@@ -631,7 +664,7 @@ wocky_xmpp_connection_recv_open_finish (WockyXmppConnection *connection,
     return FALSE;
 
   g_return_val_if_fail (g_simple_async_result_is_valid (result,
-    G_OBJECT (connection), wocky_xmpp_connection_recv_open_finish), FALSE);
+    G_OBJECT (connection), wocky_xmpp_connection_recv_open_async), FALSE);
 
   priv = connection->priv;
   priv->input_open = TRUE;
@@ -695,9 +728,10 @@ wocky_xmpp_connection_send_stanza_async (WockyXmppConnection *connection,
   g_assert (priv->output_cancellable == NULL);
 
   priv->output_result = g_simple_async_result_new (G_OBJECT (connection),
-    callback, user_data, wocky_xmpp_connection_send_stanza_finish);
+    callback, user_data, wocky_xmpp_connection_send_stanza_async);
 
-  priv->output_cancellable = cancellable;
+  if (cancellable != NULL)
+    priv->output_cancellable = g_object_ref (cancellable);
   priv->offset = 0;
   priv->length = 0;
 
@@ -750,7 +784,7 @@ wocky_xmpp_connection_send_stanza_finish (
     return FALSE;
 
   g_return_val_if_fail (g_simple_async_result_is_valid (result,
-      G_OBJECT (connection), wocky_xmpp_connection_send_stanza_finish),
+      G_OBJECT (connection), wocky_xmpp_connection_send_stanza_async),
       FALSE);
 
   return TRUE;
@@ -793,7 +827,7 @@ wocky_xmpp_connection_recv_stanza_async (WockyXmppConnection *connection,
   g_assert (priv->input_cancellable == NULL);
 
   priv->input_result = g_simple_async_result_new (G_OBJECT (connection),
-    callback, user_data, wocky_xmpp_connection_recv_stanza_finish);
+    callback, user_data, wocky_xmpp_connection_recv_stanza_async);
 
   /* There is already a stanza waiting, no need to read */
   if (wocky_xmpp_reader_peek_stanza (priv->reader) != NULL)
@@ -807,7 +841,8 @@ wocky_xmpp_connection_recv_stanza_async (WockyXmppConnection *connection,
       return;
     }
 
-  priv->input_cancellable = cancellable;
+  if (cancellable != NULL)
+    priv->input_cancellable = g_object_ref (cancellable);
 
   wocky_xmpp_connection_do_read (connection);
   return;
@@ -857,7 +892,7 @@ wocky_xmpp_connection_recv_stanza_finish (WockyXmppConnection *connection,
     return NULL;
 
   g_return_val_if_fail (g_simple_async_result_is_valid (result,
-    G_OBJECT (connection), wocky_xmpp_connection_recv_stanza_finish), NULL);
+    G_OBJECT (connection), wocky_xmpp_connection_recv_stanza_async), NULL);
 
   priv = connection->priv;
 
@@ -929,9 +964,11 @@ wocky_xmpp_connection_send_close_async (WockyXmppConnection *connection,
   g_assert (priv->output_cancellable == NULL);
 
   priv->output_result = g_simple_async_result_new (G_OBJECT (connection),
-    callback, user_data, wocky_xmpp_connection_send_close_finish);
+    callback, user_data, wocky_xmpp_connection_send_close_async);
+
+  if (cancellable != NULL)
+    priv->output_cancellable = g_object_ref (cancellable);
 
-  priv->output_cancellable = cancellable;
   priv->offset = 0;
   priv->length = 0;
 
@@ -986,7 +1023,7 @@ wocky_xmpp_connection_send_close_finish (WockyXmppConnection *connection,
     return FALSE;
 
   g_return_val_if_fail (g_simple_async_result_is_valid (result,
-    G_OBJECT (connection), wocky_xmpp_connection_send_close_finish), FALSE);
+    G_OBJECT (connection), wocky_xmpp_connection_send_close_async), FALSE);
 
   priv->output_closed = TRUE;
 
@@ -1021,7 +1058,7 @@ wocky_xmpp_connection_reset (WockyXmppConnection *connection)
 
 /**
  * wocky_xmpp_connection_new_id:
- * @connection: a #WockyXmppConnection.
+ * @self: a #WockyXmppConnection.
  *
  * Returns: A short unique string for usage as the id attribute on a stanza
  * (free after usage).
@@ -1080,7 +1117,7 @@ wocky_xmpp_connection_force_close_async (WockyXmppConnection *connection,
     }
 
   priv->force_close_result = g_simple_async_result_new (G_OBJECT (connection),
-    callback, user_data, wocky_xmpp_connection_force_close_finish);
+    callback, user_data, wocky_xmpp_connection_force_close_async);
 
   g_io_stream_close_async (priv->stream, G_PRIORITY_HIGH, cancellable,
       stream_close_cb, connection);
@@ -1097,7 +1134,7 @@ wocky_xmpp_connection_force_close_finish (
     return FALSE;
 
   g_return_val_if_fail (g_simple_async_result_is_valid (result,
-    G_OBJECT (connection), wocky_xmpp_connection_force_close_finish), FALSE);
+    G_OBJECT (connection), wocky_xmpp_connection_force_close_async), FALSE);
 
   return TRUE;
 }
