@@ -75,6 +75,9 @@ struct _GabbleConnectionPresencePrivate {
     /* Are all of the connected resources complying to version 2 */
     gboolean shared_status_compat;
 
+    /* Max length of status message */
+    gint max_status_message_length;
+
     /* Max statuses in a shared status list */
     gint max_shared_statuses;
 
@@ -823,7 +826,7 @@ store_shared_statuses (GabbleConnection *self,
   /* If we are connected, use the new shared status. If not, override with local */
   if (base->status == TP_CONNECTION_STATUS_CONNECTED)
     rv = gabble_presence_update (self->self_presence, resource, presence_id,
-        status_message, prio);
+        status_message, prio, NULL, time (NULL));
   else
     rv = TRUE;
 
@@ -947,9 +950,17 @@ get_shared_status_cb  (GObject *source_object,
 
       if (query_node != NULL)
         {
+          const gchar *max_status_message = wocky_node_get_attribute (query_node,
+              "status-max");
           const gchar *max_shared = wocky_node_get_attribute (query_node,
               "status-list-contents-max");
 
+          if (max_status_message != NULL)
+            priv->max_status_message_length = (gint) g_ascii_strtoll (
+                max_status_message, NULL, 10);
+          else
+            priv->max_status_message_length = 0; /* no limit */
+
           if (max_shared != NULL)
             priv->max_shared_statuses = (gint) g_ascii_strtoll (max_shared, NULL, 10);
           else
@@ -1437,6 +1448,15 @@ conn_presence_statuses (void)
   return gabble_statuses;
 }
 
+static guint
+get_maximum_status_message_length_cb (GObject *obj)
+{
+  GabbleConnection *conn = GABBLE_CONNECTION (obj);
+  GabbleConnectionPresencePrivate *priv = conn->presence_priv;
+
+  return priv->max_status_message_length;
+}
+
 /**
  * conn_presence_signal_own_presence:
  * @self: A #GabbleConnection
@@ -1627,6 +1647,7 @@ set_own_status_cb (GObject *obj,
   TpBaseConnection *base = (TpBaseConnection *) conn;
   GabblePresenceId i = GABBLE_PRESENCE_AVAILABLE;
   const gchar *message_str = NULL;
+  gchar *message_truncated = NULL;
   gchar *resource;
   gint8 prio;
   gboolean retval = TRUE;
@@ -1689,8 +1710,16 @@ set_own_status_cb (GObject *obj,
         }
     }
 
+  if (message_str && priv->max_status_message_length > 0 &&
+      priv->shared_statuses != NULL)
+    {
+      message_truncated = g_strndup (message_str,
+          priv->max_status_message_length);
+      message_str = message_truncated;
+    }
+
   if (gabble_presence_update (conn->self_presence, resource, i,
-          message_str, prio))
+          message_str, prio, NULL, time (NULL)))
     {
       if (base->status != TP_CONNECTION_STATUS_CONNECTED)
         {
@@ -1721,6 +1750,7 @@ set_own_status_cb (GObject *obj,
     }
 
 OUT:
+  g_free (message_truncated);
   g_free (resource);
 
   return retval;
@@ -1815,10 +1845,15 @@ conn_presence_get_type (GabblePresence *presence)
 void
 conn_presence_class_init (GabbleConnectionClass *klass)
 {
+  TpPresenceMixinClass *mixin_cls;
+
   tp_presence_mixin_class_init ((GObjectClass *) klass,
       G_STRUCT_OFFSET (GabbleConnectionClass, presence_class),
       status_available_cb, construct_contact_statuses_cb,
       set_own_status_cb, conn_presence_statuses ());
+  mixin_cls = TP_PRESENCE_MIXIN_CLASS (klass);
+  mixin_cls->get_maximum_status_message_length =
+      get_maximum_status_message_length_cb;
 
   tp_presence_mixin_simple_presence_init_dbus_properties (
     (GObjectClass *) klass);
