@@ -32,6 +32,8 @@
 #include <telepathy-glib/interfaces.h>
 #include <telepathy-glib/util.h>
 
+#include <telepathy-yell/interfaces.h>
+
 #define DEBUG_FLAG GABBLE_DEBUG_MUC
 
 #include "caps-channel-manager.h"
@@ -424,9 +426,15 @@ muc_channel_new_tube (GabbleMucChannel *channel,
     gpointer user_data)
 {
   GabbleMucFactory *fac = GABBLE_MUC_FACTORY (user_data);
+  GabbleMucFactoryPrivate *priv = fac->priv;
 
-  tp_channel_manager_emit_new_channel (fac,
+  /* If the muc channel is ready announce the tubes channel right away
+   * otherwise wait for the text channel to be ready */
+  if (_gabble_muc_channel_is_ready (channel))
+    tp_channel_manager_emit_new_channel (fac,
       TP_EXPORTABLE_CHANNEL (tube), NULL);
+  else
+    g_hash_table_insert (priv->text_needed_for_tubes, channel, tube);
 
   g_signal_connect (tube, "closed",
     G_CALLBACK (muc_sub_channel_closed_cb), fac);
@@ -825,7 +833,7 @@ gabble_muc_factory_broadcast_presence (GabbleMucFactory *self)
   while (g_hash_table_iter_next (&iter, NULL, &channel))
     {
       g_assert (GABBLE_IS_MUC_CHANNEL (channel));
-      gabble_muc_channel_send_presence (GABBLE_MUC_CHANNEL (channel), NULL);
+      gabble_muc_channel_send_presence (GABBLE_MUC_CHANNEL (channel));
     }
 }
 
@@ -884,24 +892,12 @@ gabble_muc_factory_close_all (GabbleMucFactory *self)
     }
 
   if (priv->queued_requests != NULL)
-    {
-      g_hash_table_foreach_steal (priv->queued_requests,
-          cancel_queued_requests, self);
-      g_hash_table_destroy (priv->queued_requests);
-      priv->queued_requests = NULL;
-    }
+    g_hash_table_foreach_steal (priv->queued_requests,
+        cancel_queued_requests, self);
 
-  if (priv->text_needed_for_tubes != NULL)
-    {
-      g_hash_table_destroy (priv->text_needed_for_tubes);
-      priv->text_needed_for_tubes = NULL;
-    }
-
-  if (priv->tubes_needed_for_tube != NULL)
-    {
-      g_hash_table_destroy (priv->tubes_needed_for_tube);
-      priv->tubes_needed_for_tube = NULL;
-    }
+  tp_clear_pointer (&priv->queued_requests, g_hash_table_destroy);
+  tp_clear_pointer (&priv->text_needed_for_tubes, g_hash_table_destroy);
+  tp_clear_pointer (&priv->tubes_needed_for_tube, g_hash_table_destroy);
 
   /* Use a temporary variable because we don't want
    * muc_channel_closed_cb or tubes_channel_closed_cb to remove the channel
@@ -927,9 +923,9 @@ gabble_muc_factory_close_all (GabbleMucFactory *self)
 
       lm_connection_unregister_message_handler (priv->conn->lmconn,
           priv->message_cb, LM_MESSAGE_TYPE_MESSAGE);
-      lm_message_handler_unref (priv->message_cb);
-      priv->message_cb = NULL;
     }
+
+  tp_clear_pointer (&priv->message_cb, lm_message_handler_unref);
 }
 
 
@@ -1170,7 +1166,7 @@ gabble_muc_factory_type_foreach_channel_class (GType type,
 
   /* Muc Channel.Type.Call */
   g_value_set_static_string (channel_type_value,
-      GABBLE_IFACE_CHANNEL_TYPE_CALL);
+      TPY_IFACE_CHANNEL_TYPE_CALL);
   func (type, table,
       gabble_media_factory_call_channel_allowed_properties (),
       user_data);
@@ -1212,18 +1208,17 @@ handle_text_channel_request (GabbleMucFactory *self,
                             GError **error)
 {
   GabbleMucFactoryPrivate *priv = GABBLE_MUC_FACTORY_GET_PRIVATE (self);
+  TpBaseConnection *conn = TP_BASE_CONNECTION (priv->conn);
   GabbleMucChannel *text_chan;
-
-  DBusGConnection *bus = tp_get_bus ();
   TpHandleSet *handles;
   TpIntSet *continue_handles;
   guint i;
   gboolean ret = TRUE;
 
-  TpHandleRepoIface *contact_handles = tp_base_connection_get_handles (
-      TP_BASE_CONNECTION (priv->conn), TP_HANDLE_TYPE_CONTACT);
-  TpHandleRepoIface *room_handles = tp_base_connection_get_handles (
-      TP_BASE_CONNECTION (priv->conn), TP_HANDLE_TYPE_ROOM);
+  TpHandleRepoIface *contact_handles = tp_base_connection_get_handles (conn,
+      TP_HANDLE_TYPE_CONTACT);
+  TpHandleRepoIface *room_handles = tp_base_connection_get_handles (conn,
+      TP_HANDLE_TYPE_ROOM);
 
   GPtrArray *initial_channels;
   GHashTable *final_channels; /* used as a set: (char *) -> NULL */
@@ -1255,6 +1250,9 @@ handle_text_channel_request (GabbleMucFactory *self,
   /* look at the list of initial channels, build a set of handles to invite */
   if (initial_channels != NULL)
     {
+      TpDBusDaemon *dbus_daemon = tp_base_connection_get_dbus_daemon (conn);
+      DBusGConnection *bus = tp_proxy_get_dbus_connection (dbus_daemon);
+
       for (i = 0; i < initial_channels->len; i++)
         {
           const char *object_path = g_ptr_array_index (initial_channels, i);
@@ -1367,8 +1365,6 @@ handle_text_channel_request (GabbleMucFactory *self,
             }
         }
 
-      /* N.B. gabble_generate_id() requires libuuid to generate valid UUIDs
-       * for Google PMUCs */
       uuid = gabble_generate_id ();
       id = g_strdup_printf ("private-chat-%s%s", uuid, server);
 
@@ -1712,9 +1708,9 @@ handle_call_channel_request (GabbleMucFactory *self,
     return FALSE;
 
   initial_audio = tp_asv_get_boolean (request_properties,
-      GABBLE_IFACE_CHANNEL_TYPE_CALL ".InitialAudio", NULL);
+      TPY_PROP_CHANNEL_TYPE_CALL_INITIAL_AUDIO, NULL);
   initial_video = tp_asv_get_boolean (request_properties,
-      GABBLE_IFACE_CHANNEL_TYPE_CALL ".InitialVideo", NULL);
+      TPY_PROP_CHANNEL_TYPE_CALL_INITIAL_VIDEO, NULL);
 
   if (!initial_audio && !initial_video)
     {
@@ -1798,7 +1794,7 @@ gabble_muc_factory_request (GabbleMucFactory *self,
       tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_TUBES) &&
       tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_STREAM_TUBE) &&
       tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_DBUS_TUBE) &&
-      tp_strdiff (channel_type, GABBLE_IFACE_CHANNEL_TYPE_CALL))
+      tp_strdiff (channel_type, TPY_IFACE_CHANNEL_TYPE_CALL))
     return FALSE;
 
   /* validity already checked by TpBaseConnection */
@@ -1830,7 +1826,7 @@ gabble_muc_factory_request (GabbleMucFactory *self,
           request_properties, require_new, handle, &error))
         return TRUE;
     }
-  else if (!tp_strdiff (channel_type, GABBLE_IFACE_CHANNEL_TYPE_CALL))
+  else if (!tp_strdiff (channel_type, TPY_IFACE_CHANNEL_TYPE_CALL))
     {
       if (handle_call_channel_request (self, request_token,
           request_properties, require_new, handle, &error))
