@@ -21,6 +21,7 @@
 #include "config.h"
 #include "presence-cache.h"
 #include "vcard-manager.h"
+#include "gabble-enumtypes.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -79,6 +80,7 @@ enum
   CAPABILITIES_DISCOVERED,
   LOCATION_UPDATED,
   UNSURE_PERIOD_ENDED,
+  CLIENT_TYPES_UPDATED,
   LAST_SIGNAL
 };
 
@@ -227,6 +229,7 @@ capability_info_get (GabblePresenceCache *cache, const gchar *node)
     {
       info = g_slice_new0 (GabbleCapabilityInfo);
       info->cap_set = NULL;
+      info->client_types = 0;
       info->guys = tp_intset_new ();
       g_hash_table_insert (priv->capabilities, g_strdup (node), info);
     }
@@ -256,7 +259,8 @@ capability_info_recvd (GabblePresenceCache *cache,
     const gchar *node,
     TpHandle handle,
     GabbleCapabilitySet *cap_set,
-    guint trust_inc)
+    guint trust_inc,
+    guint client_types)
 {
   GabbleCapabilityInfo *info = capability_info_get (cache, node);
 
@@ -285,6 +289,8 @@ capability_info_recvd (GabblePresenceCache *cache,
       info->trust += trust_inc;
     }
 
+  info->client_types = client_types;
+
   return info->trust;
 }
 
@@ -417,6 +423,14 @@ gabble_presence_cache_class_init (GabblePresenceCacheClass *klass)
     NULL, NULL,
     g_cclosure_marshal_VOID__VOID, G_TYPE_NONE,
     0);
+
+  signals[CLIENT_TYPES_UPDATED] = g_signal_new (
+    "client-types-updated",
+    G_TYPE_FROM_CLASS (klass),
+    G_SIGNAL_RUN_LAST,
+    0,
+    NULL, NULL,
+    g_cclosure_marshal_VOID__UINT, G_TYPE_NONE, 1, TP_TYPE_HANDLE);
 }
 
 static gboolean
@@ -1122,6 +1136,7 @@ static void
 set_caps_for (DiscoWaiter *waiter,
     GabblePresenceCache *cache,
     GabbleCapabilitySet *cap_set,
+    guint client_types,
     TpHandle responder_handle,
     const gchar *responder_jid)
 {
@@ -1139,12 +1154,13 @@ set_caps_for (DiscoWaiter *waiter,
 
   gabble_presence_set_capabilities (presence, waiter->resource, cap_set,
       waiter->serial);
-
   new_cap_set = gabble_presence_peek_caps (presence);
-
   emit_capabilities_update (cache, waiter->handle, old_cap_set, new_cap_set);
-
   gabble_capability_set_free (old_cap_set);
+
+  if (gabble_presence_update_client_types (presence, waiter->resource,
+        client_types))
+    g_signal_emit (cache, signals[CLIENT_TYPES_UPDATED], 0, waiter->handle);
 }
 
 static void
@@ -1154,31 +1170,24 @@ emit_capabilities_discovered (GabblePresenceCache *cache,
   g_signal_emit (cache, signals[CAPABILITIES_DISCOVERED], 0, handle);
 }
 
-static GPtrArray *
+static guint
 client_types_from_message (TpHandle handle,
     LmMessageNode *lm_node,
     const gchar *resource)
 {
   WockyNode *identity, *query_result = (WockyNode *) lm_node;
   WockyNodeIter iter;
-  GPtrArray *array;
-
-  array = g_ptr_array_new_with_free_func (g_free);
+  guint client_types = 0;
 
   /* Find all identity nodes in the result. */
-  wocky_node_iter_init (&iter, query_result,
-      "identity", NS_DISCO_INFO);
+  wocky_node_iter_init (&iter, query_result, "identity", NS_DISCO_INFO);
   while (wocky_node_iter_next (&iter, &identity))
     {
-      const gchar *category, *type;
+      const gchar *category = wocky_node_get_attribute (identity, "category");
+      const gchar *type = wocky_node_get_attribute (identity, "type");
+      guint value;
 
-      category = wocky_node_get_attribute (identity, "category");
-      if (category == NULL)
-        continue;
-
-      /* Now get the client type */
-      type = wocky_node_get_attribute (identity, "type");
-      if (type == NULL)
+      if (category == NULL || type == NULL)
         continue;
 
       /* So, turns out if you disco a specific resource of a gtalk
@@ -1192,25 +1201,20 @@ client_types_from_message (TpHandle handle,
       they're phones. */
 
       if (!tp_strdiff (category, "account")
-          && g_str_has_prefix (resource, "android")
+          && (resource != NULL && g_str_has_prefix (resource, "android"))
           && !tp_strdiff (type, "registered"))
         {
-          type = "phone";
+          client_types |= GABBLE_CLIENT_TYPE_PHONE;
+        }
+      else if (!tp_strdiff (category, "client") &&
+          gabble_flag_from_nick (GABBLE_TYPE_CLIENT_TYPE, type, &value))
+        {
+          DEBUG ("Got type for %u: %s (%u)", handle, type, value);
+          client_types |= value;
         }
-
-      DEBUG ("Got type for %u: %s", handle, type);
-
-      g_ptr_array_add (array, g_strdup (type));
     }
 
-  if (array->len == 0)
-    {
-      DEBUG ("How very odd, we didn't get any client types");
-      g_ptr_array_unref (array);
-      return NULL;
-    }
-
-  return array;
+  return client_types;
 }
 
 static void
@@ -1225,36 +1229,6 @@ _signal_presences_updated (GabblePresenceCache *cache,
   g_array_free (handles, TRUE);
 }
 
-static void
-process_client_types (
-    GabblePresenceCache *cache,
-    LmMessageNode *query_result,
-    TpHandle handle,
-    DiscoWaiter *waiter_self)
-{
-  GabblePresence *presence = gabble_presence_cache_get (cache, handle);
-  GPtrArray *client_types;
-
-  /* If the contact's gone offline since we sent the disco request, we have no
-   * presence to attach their freshly-discovered client types to.
-   */
-  if (presence == NULL)
-    return;
-
-  client_types = client_types_from_message (handle, query_result,
-      waiter_self->resource);
-
-  if (waiter_self->resource != NULL)
-    gabble_presence_update_client_types (presence, waiter_self->resource,
-        client_types);
-
-  if (client_types != NULL)
-    {
-      g_ptr_array_unref (client_types);
-      _signal_presences_updated (cache, handle);
-    }
-}
-
 static void
 _caps_disco_cb (GabbleDisco *disco,
                 GabbleDiscoRequest *request,
@@ -1277,6 +1251,7 @@ _caps_disco_cb (GabbleDisco *disco,
   gchar *resource;
   gboolean jid_is_valid;
   gpointer key;
+  guint client_types = 0;
 
   cache = GABBLE_PRESENCE_CACHE (user_data);
   priv = cache->priv;
@@ -1321,10 +1296,10 @@ _caps_disco_cb (GabbleDisco *disco,
       goto OUT;
     }
 
-  process_client_types (cache, query_result, handle, waiter_self);
-
   /* Now onto caps */
   cap_set = gabble_capability_set_new_from_stanza (query_result);
+  client_types = client_types_from_message (handle, query_result,
+      waiter_self->resource);
 
   /* Only 'sha-1' is mandatory to implement by XEP-0115. If the remote contact
    * uses another hash algorithm, don't check the hash and fallback to the old
@@ -1340,7 +1315,7 @@ _caps_disco_cb (GabbleDisco *disco,
       if (g_str_equal (waiter_self->ver, computed_hash))
         {
           trust = capability_info_recvd (cache, node, handle, cap_set,
-              CAPABILITY_BUNDLE_ENOUGH_TRUST);
+              CAPABILITY_BUNDLE_ENOUGH_TRUST, client_types);
         }
       else
         {
@@ -1355,7 +1330,7 @@ _caps_disco_cb (GabbleDisco *disco,
     }
   else
     {
-      trust = capability_info_recvd (cache, node, handle, cap_set, 1);
+      trust = capability_info_recvd (cache, node, handle, cap_set, 1, client_types);
     }
 
   /* Remove the node from the hash table without freeing the key or list of
@@ -1395,7 +1370,7 @@ _caps_disco_cb (GabbleDisco *disco,
         {
           DiscoWaiter *waiter = (DiscoWaiter *) i->data;
 
-          set_caps_for (waiter, cache, cap_set, handle, jid);
+          set_caps_for (waiter, cache, cap_set, client_types, handle, jid);
           emit_capabilities_discovered (cache, waiter->handle);
         }
 
@@ -1423,7 +1398,7 @@ _caps_disco_cb (GabbleDisco *disco,
               g_free (tmp);
             }
 
-          set_caps_for (waiter_self, cache, cap_set, handle, jid);
+          set_caps_for (waiter_self, cache, cap_set, client_types, handle, jid);
         }
 
       waiters = g_slist_remove (waiters, waiter_self);
@@ -1512,6 +1487,8 @@ _process_caps_uri (GabblePresenceCache *cache,
 
       if (presence)
         {
+          guint types;
+
           gabble_presence_set_capabilities (
               presence, resource, cap_set, serial);
 
@@ -1520,18 +1497,15 @@ _process_caps_uri (GabblePresenceCache *cache,
           if (cached_query_reply != NULL)
             {
               WockyNode *query = wocky_node_tree_get_top_node (cached_query_reply);
-              GPtrArray *types = client_types_from_message (handle, query, resource);
-
-              if (resource != NULL)
-                gabble_presence_update_client_types (presence, resource, types);
-
-              if (types != NULL)
-                {
-                  g_ptr_array_unref (types);
-
-                  _signal_presences_updated (cache, handle);
-                }
+              types = client_types_from_message (handle, query, resource);
             }
+          else
+            {
+              types = info->client_types;
+            }
+
+          if (gabble_presence_update_client_types (presence, resource, types))
+            g_signal_emit (cache, signals[CLIENT_TYPES_UPDATED], 0, handle);
         }
       else
         DEBUG ("presence not found");
@@ -1977,7 +1951,8 @@ gabble_presence_cache_do_update (
     const gchar *resource,
     GabblePresenceId presence_id,
     const gchar *status_message,
-    gint8 priority)
+    gint8 priority,
+    gboolean *update_client_types)
 {
   GabblePresenceCachePrivate *priv = cache->priv;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
@@ -2003,7 +1978,8 @@ gabble_presence_cache_do_update (
   old_cap_set = gabble_presence_dup_caps (presence);
 
   ret = gabble_presence_update (presence, resource, presence_id,
-      status_message, priority);
+      status_message, priority, update_client_types,
+      time (NULL));
 
   new_cap_set = gabble_presence_peek_caps (presence);
 
@@ -2023,12 +1999,17 @@ gabble_presence_cache_update (
     const gchar *status_message,
     gint8 priority)
 {
+  gboolean update_client_types = FALSE;
+
   if (gabble_presence_cache_do_update (cache, handle, resource, presence_id,
-      status_message, priority))
+          status_message, priority, &update_client_types))
     {
       _signal_presences_updated (cache, handle);
     }
 
+  if (update_client_types)
+    g_signal_emit (cache, signals[CLIENT_TYPES_UPDATED], 0, handle);
+
   gabble_presence_cache_maybe_remove (cache, handle);
 }
 
@@ -2054,7 +2035,7 @@ gabble_presence_cache_update_many (
       handle = g_array_index (contact_handles, TpHandle, i);
 
       if (gabble_presence_cache_do_update (cache, handle, resource,
-          presence_id, status_message, priority))
+          presence_id, status_message, priority, NULL))
         {
           g_array_append_val (updated, handle);
         }
@@ -2468,23 +2449,27 @@ gabble_presence_cache_disco_in_progress (GabblePresenceCache *cache,
     const gchar *resource)
 {
   GabblePresenceCachePrivate *priv = cache->priv;
-  GList *l, *waiters;
-  gboolean out = FALSE;
+  GList *l, *waiter_list;
+  gboolean in_progress = FALSE;
 
-  waiters = g_hash_table_get_values (priv->disco_pending);
+  waiter_list = g_hash_table_get_values (priv->disco_pending);
 
-  for (l = waiters; l != NULL; l = l->next)
+  for (l = waiter_list; !in_progress && l != NULL; l = l->next)
     {
-      DiscoWaiter *w = l->data;
+      GList *j;
 
-      if (w != NULL && w->handle == handle && !tp_strdiff (w->resource, resource))
+      for (j = l->data; !in_progress && j != NULL; j = j->next)
         {
-          out = TRUE;
-          break;
+          DiscoWaiter *w = j->data;
+
+          if (w != NULL &&
+              w->handle == handle &&
+              !tp_strdiff (w->resource, resource))
+            in_progress = TRUE;
         }
     }
 
-  g_list_free (waiters);
+  g_list_free (waiter_list);
 
-  return out;
+  return in_progress;
 }
