@@ -25,13 +25,9 @@ def echo_muc_presence (q, stream, stanza, affiliation, role):
 
     stream.send (stanza)
 
-def join_muc(q, bus, conn, stream, muc, request=None,
-        also_capture=[], role='participant'):
-    """
-    Joins 'muc', returning the muc's handle, a proxy object for the channel,
-    its path and its immutable properties just after the CreateChannel event
-    has fired. The room contains one other member.
-    """
+def try_to_join_muc(q, bus, conn, stream, muc, request=None):
+    """Ask Gabble to join a MUC, and expect it to send <presence/>"""
+
     if request is None:
         request = {
             cs.CHANNEL_TYPE: cs.CHANNEL_TYPE_TEXT,
@@ -39,8 +35,6 @@ def join_muc(q, bus, conn, stream, muc, request=None,
             cs.TARGET_ID: muc,
         }
 
-    muc_handle = request_muc_handle(q, conn, stream, muc)
-
     requests = dbus.Interface(conn, cs.CONN_IFACE_REQUESTS)
     call_async(q, requests, 'CreateChannel',
         dbus.Dictionary(request, signature='sv'))
@@ -54,6 +48,16 @@ def join_muc(q, bus, conn, stream, muc, request=None,
         join_event.stanza)
     assertLength(1, x_muc_nodes)
 
+def join_muc(q, bus, conn, stream, muc, request=None,
+        also_capture=[], role='participant'):
+    """
+    Joins 'muc', returning the muc's handle, a proxy object for the channel,
+    its path and its immutable properties just after the CreateChannel event
+    has fired. The room contains one other member.
+    """
+    muc_handle = request_muc_handle(q, conn, stream, muc)
+    try_to_join_muc(q, bus, conn, stream, muc, request=request)
+
     # Send presence for other member of room.
     stream.send(make_muc_presence('owner', 'moderator', muc, 'bob'))
 
